#ifndef RASTER_H
#define RASTER_H

#ifdef __cplusplus
extern "C" {
#endif

typedef struct Screen {
	int w, h;
	int displayable;
	unsigned short *pixels;
} Screen;
	
/* pixelsize:
 * 0 = 565
 * 1 = 5551
 * 2 = 4444
 * 3 = 8888
 */

#define LINELENGTH 1024
#define PIXELSIZE 2
#define FRAMESIZE 0x44000

#define SCR_WIDTH 480
#define SCR_HEIGHT 272

// x * 512 == x << 9
// #define PIXEL(screen, x, y, col) frame[screen][((y) << 9) + (x)] = col
#define PIXEL_S(x, y, col) target->pixels[((y) << 9) + (x)] = col
#define PIXEL(dest, x, y, col) dest->pixels[(y * dest->w) + x] = col

void rast_init();

//create a new screen
Screen *rast_create_screen(int w, int h);
void rast_destroy_screen(Screen *scr);

// flip buffer over
void rast_flip();

//fill buffer with one colour
void rast_fill(Screen *tgt, unsigned short col);
void rast_blank_z();

//draw a pixel to the current buffer
inline void rast_pixel(Screen *tgt, int x, int y, unsigned short col);

//rectangle
void rast_rect(Screen *tgt, const unsigned int x, const unsigned int y,
		const unsigned int w, const unsigned int h,
		const unsigned short col);

//blit
void rast_blit(Screen *dst, Screen *src, int dx, int dy, int sx, int sy,
		int sw, int sh);
void rast_blit_alpha(Screen *dst, Screen *src, int dx, int dy, int sx, int sy,
		int sw, int sh);
void rast_blit_scale(Screen *dst, Screen *src, int dx, int dy, int dw, int dh,
		int sx, int sy, int sw, int sh);


//text
void rast_drawtext(Screen *dst, int x,int y, unsigned short colour, char *text);

//line
void rast_line(Screen *dst, int sx, int sy, int dx, int dy, unsigned short col);

//polygon
void rast_vertex(int x, int y);
void rast_coord(int x, int y, float tcx, float tcy, float zc);
void rast_poly_line(Screen *dst, unsigned short col); //wireframe
void rast_poly_fill(Screen *dst, unsigned short col); //filled
void rast_poly_textured(Screen *dst);

#ifdef __cplusplus
}
#endif
#endif // RASTER_H
